//@author: Martin Zrcek
//@help: This is a background substraction with some filters to reduce camera noise (looks to pixels around)
//@tags:
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
//	Stored Texture ... is an image substracted from Camera texture
//	Cross Span ... how many pixels should it look around
//	Pass Level ... lower bound of what is a different picture and should be shown
//	Softness ... 
// --------------------------------------------------------------------------------------------------

//texture
texture Tex1 <string uiname="Stored Texture";>;
sampler StoredSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex1);          //apply a texture to the sampler
    MipFilter = none;         //sampler states
    MinFilter = none;
    MagFilter = none;
};
//texture
texture Tex2 <string uiname="Camera Texture";>;
sampler CameraSamp = sampler_state  
{
    Texture   = (Tex2);   
    MipFilter = linear;   
    MinFilter = linear;
    MagFilter = linear;
};
// previous frame
texture Tex3 <string uiname="Previous Texture (only for KinectRedPass)";>;
sampler PreviousSamp = sampler_state  
{
    Texture   = (Tex3);   
    MipFilter = none;  
    MinFilter = none;
    MagFilter = none;
};
float passLevel = 0.1;
float crossSpan = 1;
float softness = .05;
int backbufferWidth = 400;	// this is needed because of size of 1 pixel on screen
int backbufferHeight = 300;	// to know how far another pixel is

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //declare output struct
    vs2ps Out;

    //transform position
    Out.Pos = float4(PosO.x*2.0-(1.0/backbufferWidth), PosO.y*2.0+(1.0/backbufferHeight), 0.0, 1.0);
    //Out.Pos = float4(PosO.x*2.0, PosO.y*2.0, 0.0, 1.0);
    
    //transform texturecoordinates
    Out.TexCd = TexCd;

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col0 = tex2D(StoredSamp, In.TexCd);
    float4 col1 = tex2D(CameraSamp, In.TexCd);
	float4 col = col1;
	if (length(col0-col1)>passLevel) col=col1;
	else col.a=0;
    return col;
}
float4 PScross(vs2ps In): COLOR
{
	float4 col = tex2D(CameraSamp, In.TexCd);
	float2 cSpanX = float2(crossSpan / backbufferWidth,0.0);
	float2 cSpanY = float2(0.0,crossSpan / backbufferHeight);
	int stencil=0;
    if(length( tex2D(StoredSamp, In.TexCd) - tex2D(CameraSamp, In.TexCd) )>passLevel) stencil+=2;
    if(length( tex2D(StoredSamp, In.TexCd+cSpanX) - tex2D(CameraSamp, In.TexCd+cSpanX) )>passLevel) stencil++;
    if(length( tex2D(StoredSamp, In.TexCd+cSpanY) - tex2D(CameraSamp, In.TexCd+cSpanY) )>passLevel) stencil++;
    if(length( tex2D(StoredSamp, In.TexCd-cSpanX) - tex2D(CameraSamp, In.TexCd-cSpanX) )>passLevel) stencil++;
    if(length( tex2D(StoredSamp, In.TexCd-cSpanY) - tex2D(CameraSamp, In.TexCd-cSpanY) )>passLevel) stencil++;
	if (stencil>2) col.a=1.0;
	else col.a=.0;
    return col;
}
// looks at a place and sums distances form 5 positions in X
float examine (float2 place) {
	float mult = 0.4;
	float2 XSpan1 = float2(mult*crossSpan / backbufferWidth,mult*crossSpan / backbufferHeight);
	float2 XSpan2 = float2(-mult*crossSpan / backbufferWidth,mult*crossSpan / backbufferHeight);
	float diffSum = 0;
	diffSum+= length( tex2D(StoredSamp, place).xyz - tex2D(CameraSamp, place).xyz );
	diffSum+= length( tex2D(StoredSamp, place+XSpan1).xyz - tex2D(CameraSamp, place+XSpan1).xyz );
	diffSum+= length( tex2D(StoredSamp, place+XSpan2).xyz - tex2D(CameraSamp, place+XSpan2).xyz );
	diffSum+= length( tex2D(StoredSamp, place-XSpan1).xyz - tex2D(CameraSamp, place-XSpan1).xyz );
	diffSum+= length( tex2D(StoredSamp, place-XSpan2).xyz - tex2D(CameraSamp, place-XSpan2).xyz );
	return diffSum*.2;
}
float4 PSfloatCross(vs2ps In): COLOR
{
	float4 col = tex2D(CameraSamp, In.TexCd);
	float2 cSpanX = float2(crossSpan / backbufferWidth,0.0);
	float2 cSpanY = float2(0.0,crossSpan / backbufferHeight);
	float sum= length( tex2D(StoredSamp, In.TexCd) - tex2D(CameraSamp, In.TexCd) );
	sum+= examine(In.TexCd+cSpanX);
	sum+= examine(In.TexCd+cSpanY);
	sum+= examine(In.TexCd-cSpanX);
	sum+= examine(In.TexCd-cSpanY);
	sum*=.2;
	if (sum>=passLevel+softness) col.a=1.0;
	else if (sum<=passLevel) col.a=0.0;
	else col.a = (sum-passLevel)/softness;
	//col.a=1;
    return col;
}
float4 PSKinectRed(vs2ps In): COLOR
{
	float4 col = tex2D(CameraSamp, In.TexCd);
	if (col.r>0.01) col.r = 1.0 - col.r;
	col.gb = tex2D(PreviousSamp, In.TexCd).gb;	
		// in .r is this frame, .g is previous frame and .b is the one before
	float2 cSpanX = float2(crossSpan / backbufferWidth,0.0);
	float2 cSpanY = float2(0.0,crossSpan / backbufferHeight);
	
	float stored = tex2D(StoredSamp, In.TexCd).r; //.g.b.a not important
	float sum = 0;
	if ((col.r - stored) > passLevel) sum+=4;
	if ((col.g - stored) > passLevel) sum+=4;
	if ((col.b - stored) > passLevel) sum+=4;
	
	float nearby = tex2D(CameraSamp, In.TexCd+cSpanX).r;
	if (nearby>0.01) nearby = 1.0 - nearby;
	if ( nearby - tex2D(StoredSamp, In.TexCd+cSpanX).r > passLevel ) sum++;
	nearby = tex2D(CameraSamp, In.TexCd+cSpanY).r;
	if (nearby>0.01) nearby = 1.0 - nearby;
	if ( nearby - tex2D(StoredSamp, In.TexCd+cSpanY).r > passLevel ) sum++;
	nearby = tex2D(CameraSamp, In.TexCd-cSpanX).r;
	if (nearby>0.01) nearby = 1.0 - nearby;
	if ( nearby - tex2D(StoredSamp, In.TexCd-cSpanX).r > passLevel ) sum++;
	nearby = tex2D(CameraSamp, In.TexCd-cSpanY).r;
	if (nearby>0.01) nearby = 1.0 - nearby;
	if ( nearby - tex2D(StoredSamp, In.TexCd-cSpanY).r > passLevel ) sum++;
	
	//return sum;
	//if (sum>0.001) return 1; else return 0;
	
		// to next frame
		// ! in output image .r is different from .g.b !
	col.gb=col.rg;
	
	sum/=16.0;
	if (sum>softness) col.r=1.0; else col.r=0.0;
	/*if (sum>=passLevel+softness) col.r=1.0;
	else if (sum<=passLevel) col.r=0.0;
	else col.r = (sum-passLevel)/softness;*/
	col.a=1.0;//col.r;
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TSimpleHighPass
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}
technique TCrossHighPass
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PScross();
    }
}
technique TCrossFloatPass
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PSfloatCross();
    }
}
technique TKinectRedPass
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PSKinectRed();
    }
}