//@author: Martin Zrcek
//@help: Contour around shapes (pixel shader)
//@tags:
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

float contourScale = 10.0;
float strength = 1.0;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //declare output struct
    vs2ps Out;

    //transform position
    Out.Pos = float4(PosO.x*2.0, PosO.y*2.0, 0.0, 1.0);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
	contourScale*=0.001;
	 float4 col1 = tex2D(Samp, float2(In.TexCd.x+contourScale,In.TexCd.y));
	 float4 col2 = tex2D(Samp, float2(In.TexCd.x,In.TexCd.y+contourScale));
	 float4 col3 = tex2D(Samp, float2(In.TexCd.x-contourScale,In.TexCd.y));
	 float4 col4 = tex2D(Samp, float2(In.TexCd.x,In.TexCd.y-contourScale));
    //float difference = ( length(col1-col2) + length(col2-col3) + length(col3-col4) + length(col4-col1) );
	float difference = ( length(col1.a*col1.rgb-col2.a*col2.rgb) + length(col2.a*col2.rgb-col3.a*col3.rgb) + length(col3.a*col3.rgb-col4.a*col4.rgb) + length(col4.a*col4.rgb-col1.a*col1.rgb) );
	float4 col = strength*difference;//tex2D(Samp, In.TexCd);
	//col+=col1+col2+col3+col4; col = col/5.0;
	col.a += (col1.a+col2.a+col3.a+col4.a)*.1;
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TSimpleShader
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}