//@author: Martin Zrcek
//@help: Effect slowly changes the output picture. It just simply adds new picture to old one in some "speed".
//@tags:
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS: 
//		backbufferWidth, Height: Should be the resolution of final image
//		speeed: <0..2> How fast will the final image update
//		addContrast: Enable function smoothstep for rgb values
// --------------------------------------------------------------------------------------------------

//texture
texture Tex1 <string uiname="Texture actual";>;
sampler ActualSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex1);          //apply a texture to the sampler
    MipFilter = point;         //sampler states
    MinFilter = point;
    MagFilter = point;
};
texture Tex2 <string uiname="Texture stored";>;
sampler StoredSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = point;         //sampler states
    MinFilter = point;
    MagFilter = point;
};
int backbufferWidth = 400;
int backbufferHeight = 300;
float speed = 0.1;
bool addContrast = false;
bool insert = true;


//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //declare output struct
    vs2ps Out;

    //transform position
    Out.Pos = float4(PosO.x*2.0-(1.0/backbufferWidth), PosO.y*2.0+(1.0/backbufferHeight), 0.0, 1.0);
    
    //transform texturecoordinates
    Out.TexCd = TexCd;

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
	if (insert) {
		float4 col = tex2D(ActualSamp, In.TexCd);
		if (addContrast) col = 3.0*col*col - 2.0*col*col*col;
	    col = speed*col+tex2D(StoredSamp, In.TexCd);
	    col.rgba/=1.0+speed;
		//col.a = 1.5;
		return col;
	}
	else {
		float4 col = tex2D(StoredSamp, In.TexCd);
		if (addContrast) col = 3.0*col*col - 2.0*col*col*col;
		return col;
	}
}
float4 PSTrace(vs2ps In): COLOR
{
	float4 col = tex2D(ActualSamp, In.TexCd);
	if (addContrast) col = 3.0*col*col - 2.0*col*col*col;
    col = col+(1.0-speed)*tex2D(StoredSamp, In.TexCd);
    //col.rgba/=2.0-speed;
	//col.a = 1.5;
	return col;
}
float4 PSKinect(vs2ps In): COLOR
{
	if (insert) {
		float4 col = tex2D(ActualSamp, In.TexCd);
		if (col.r>0.01) col.r=1.0-col.r;
		if (addContrast) col = 3.0*col*col - 2.0*col*col*col;
		col.gb=0.0;
		float4 stored = tex2D(StoredSamp, In.TexCd);
		speed = saturate(speed*2.0);
		if (col.r<stored.r) speed*=.02;
	    col = speed*col+stored;
	    col.rgba/=1.0+speed;
		//col.a = 1.5;
		return col;
	}
	else {
		float4 col = tex2D(StoredSamp, In.TexCd);
		if (addContrast) col = 3.0*col*col - 2.0*col*col*col;
		return col;
	}
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TMakeAvaredge
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}
technique TMakeTrace
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PSTrace();
    }
}
technique TStoreKinectNearest
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PSKinect();
    }
}

