//@author: Martin Zrcek
//@help: this shader generally draws a color texture flat-shaded by a mesh. It is supposed to render a Delaunay triangulated mesh.
//@tags:
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
//		mixTriangulationEfect: <0..1> show original picture
//		searchStep: How big steps (in pixels) should the search for contour do
//		searchLimit: Where is the limit
//		iterateNtimes: how many times to search
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;
float mixTriangulationEffect;
float searchStep = 1;
float searchLimit = 0.9;
int iterateNtimes = 50;

//texture
texture Tex1 <string uiname="ColorTexture";>;
sampler ColorSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex1);          //apply a texture to the sampler
    MipFilter = none;         //sampler states
    MinFilter = none;
    MagFilter = none;
};
//contour
texture Tex2 <string uiname="ContourTexture";>;
sampler ContourSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = none;         //sampler states
    MinFilter = none;
    MagFilter = none;
};

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
	//float2 TexCdColor : TEXCOORD1;
	float4 Col : COLOR0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
	float IdVertex : TEXCOORD1)
{
    //declare output struct
    vs2ps Out;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //color
	Out.TexCd = float2(PosO.x*.5+.5,PosO.y*-.5+.5);
	Out.Col = tex2Dlod(ColorSamp, float4(PosO.x*.5+.5,PosO.y*-.5+.5,0,0));

    return Out;
}
vs2ps VSmovable(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
	float IdVertex : TEXCOORD1)
{
    //declare output struct
    vs2ps Out;
	searchStep*=0.001;

    //transform position
	float4 vertexPosition = float4(PosO.x*.5+.5,PosO.y*-.5+.5,.0,.0);
	for (int i=0; i<iterateNtimes; i++) 
		if ( tex2Dlod(ContourSamp, vertexPosition).r<searchLimit ) {
			vertexPosition.y+= searchStep;
			//break;
		}
	
	
    Out.Pos = float4(vertexPosition.x*2.0-1.0, vertexPosition.y*-2.0+1.0, 0.0, 1.0);   
    //color
	Out.TexCd = float2(vertexPosition.x,vertexPosition.y);
	Out.Col = tex2Dlod(ColorSamp, vertexPosition);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col = mixTriangulationEffect*tex2D(ColorSamp, In.TexCd) + (1-mixTriangulationEffect)*In.Col;
    //col.a = 2.0*In.Col.a;
	return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique DrawTriangulated
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    	//fillmode = solid;
    	shademode = flat;
    }
}
technique DrawTriangulatedEdges
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    	fillmode = wireframe;
    	//shademode = flat;
    }
}
technique MoveVertexOnContourAndDrawTriangulated
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VSmovable();
        PixelShader  = compile ps_3_0 PS();
    	//fillmode = wireframe;
    	shademode = flat;
    }
}
technique MoveVertexOnContourAndDrawTriangulatedEdges
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VSmovable();
        PixelShader  = compile ps_3_0 PS();
    	fillmode = wireframe;
    	//shademode = flat;
    }
}